import os, shutil, logging
from trview import ufiles, locate_dir

logger = logging.getLogger('trview.astra_exp')


def astra_exp(nshot, ion_d, ext_d, nbi):

    f_exp = 'aug%d' %nshot
    udb = '%s/udb/%d' %(os.getenv('HOME'), nshot) 
    awd = os.getenv('AWD')
    if awd is None:
        awd = locate_dir.locate('.tsk')
    if awd is None:
        awd = locate_dir.locate_rec('.tsk')
    as_udb = '%s/udb/%d' %(awd, nshot)
    as_exp = '%s/exp' %awd
    as_nbi = '%s/nbi' %as_exp
    os.system('mkdir -p %s' %as_udb)
    os.system('mkdir -p %s' %as_nbi)

    atxt1 =  'aug #%d Name	 Time	 Value	 Error\n' %nshot
    atxt1 += """
|____|--|____|--|____|--|____|----------------
NA1             91
RTOR            1.65
AWALL           0.7              Reserved minor radius in the mid-plane
ELONM           1.70             Chamber elongation
TRICH           0.15             Chamber traingularity
AB              0.6              Chamber minor radius in the mid-plane
"""

    atxt1 += 'ZMJ             %3.1f              Majority Z\n' %float(ion_d['Zmain'])
    atxt1 += 'AMJ             %3.1f              Majority mass/m_p\n' %float(ion_d['Amain'])

    atxt1 += 'IPL      U-file:%d/MAG%d.IPL\n' %(nshot, nshot)
    atxt1 += 'BTOR     U-file:%d/MAG%d.BTOR\n'   %(nshot, nshot)

    atxt2 = '\nFILTER   0.002\n\n'
    uf_d = {'E': ('TE', '1.e-3'), 'I': ('TI', '1.e-3'), 'N': ('NE', '1.e-13'), \
            'V': ('VTOR', '1.e-2') , 'ANGF': ('CAR1', '1.'), 'Q': ('CAR2', '1.') }

    for prefix in ['Q'] + list(ext_d.keys()):
        if prefix == 'Q':
            ext = 'QPR'
        else:
            ext = ext_d[prefix]
        fudb = '%s%d.%s' %(prefix, nshot, ext)
        source = '%s/%s' %(udb, fudb)
        logger.debug('%s', fudb)
        if os.path.isfile(source):
            target = '%s/%s' %(as_udb, fudb)
            uf = ufiles.UFILE(fin=source)
            rho_lbl = uf.Y['label'].strip()
            if rho_lbl == 'rho_tor':
                logger.info('cp %s %s', source, target)
                shutil.copy2(source, target)
                atxt2 += '%-4s     U-file:%d/%s:%s\n'  %(uf_d[prefix][0], nshot, fudb, uf_d[prefix][1])
            else:
                logger.error('u-file label %s in %s is not supported (only rho_tor)', rho_lbl, source)
        else:
            logger.error('File %s not found', source)

    atxt2 += '\n'

# Create ASTRA exp file

    fexp = '%s/%s' %(as_exp, f_exp)
    if os.path.isfile(fexp):
        fbak = '%s~' %fexp
        logger.info('Creating backup file %s' %fbak)
        shutil.move(fexp, fbak)
    f = open(fexp, 'w')
    f.write(atxt1)
    f.write(atxt2)

    f.write('NAMEXP BNDU\n')
    f.write('../udb/%d/%d.bnd.txt\n' %(nshot, nshot))

    f.close()
    logger.info('Written %s' %fexp)

# Create exp/nbi file

    if nbi.status:
        nbi.tr2as()
        f_nbi = '%s/%s' %(as_nbi, f_exp)
        logger.info('Writing %s' %f_nbi)
        if os.path.isfile(f_nbi):
            shutil.copy2(f_nbi, '%s~' %f_nbi)
        fmt = ' %11.4e'
        as_nbi = '!\n'
        for jx, einj in enumerate(nbi.einj_kev):
            jnb = jx + 1
            as_nbi += '  %d\n' %jnb
            as_nbi += 'ZRD%d        ' %jnb + 3*fmt %(0, 2, 1)
            en = 1.e-3*nbi.einja[jx]
            as_nbi += fmt %einj
            as_nbi += '\n'
            for spec in nbi.spec_arr[:, jx]:
                as_nbi += fmt %spec
            as_nbi += '  1.0000E+00  8.1000E+01'
            as_nbi += '\n'
            as_nbi += fmt  %nbi.hbeam[jx]
            as_nbi += fmt  %nbi.rbtmax[jx]
            as_nbi += fmt  %nbi.rbtmin[jx]
            as_nbi += fmt  %nbi.tgA[jx]
            as_nbi += fmt  %nbi.aspect
            as_nbi += '\n'
            as_nbi += 5*fmt %(4, 2, 4, 2, 0)
            as_nbi += '\n'

        fnbi = open(f_nbi, 'w')
        fnbi.write(as_nbi)
        fnbi.close()


if __name__ == '__main__':

    nshot = 28053
    sfl_d = {'I': ['AUGD:CEZ:Ti_c:0', 'AUGD:CMZ:Ti_c:0'], \
             'V': ['AUGD:CEZ:vrot:0', 'AUGD:CMZ:vrot:0'], \
             'E': ['AUGD:IDA:Te:0'], \
             'N': ['AUGD:IDA:ne:0']  }
    ion_d = {'Amain': 2, 'Zmain': 1, 'Aimp': 12, 'Zimp': 6}
    astra_exp(nshot, ion_d, sfl_d)
