import os, logging
import numpy as np
from multiprocessing import Pool, cpu_count
import aug_sfutils as sf
from trview import descur2d, mom2rz, time_no_elm

logger = logging.getLogger('trview.fit_sep')


class geofit:


    def __init__(self, equ, nmom=6, n_the=51):

        descu = descur2d.scrunch

        inherit = ('Rmag', 'Zmag', 'Rzunt', 'Zunt', 'Rzoben', 'Zoben', \
            'Rxpu', 'Zxpu', 'Rxpo', 'Zxpo', 'rho_tor_n', 'q', \
            'time', 'shot', 'exp', 'diag', 'ed')

        for key in inherit:
            self.__dict__[key] = equ.__dict__[key]

        self.rho_sep = 0.999
        self.rscat, self.zscat = sf.rho2rz(equ, self.rho_sep)

        nt_in = len(self.time)
        self.error = np.zeros(nt_in, dtype=np.int32)
        ind_elm = time_no_elm.time_no_elm(self.shot, self.time)
        self.error[ind_elm] = 1
        for jt in range(nt_in):
            if (max(self.rscat[jt][0]) > 2.2):
                self.error[jt] = 2
            elif (min(self.zscat[jt][0]) < -1.1):
                self.error[jt] = 3
            elif (max(self.zscat[jt][0]) > 1.2):
                self.error[jt] = 4

        logger.info('Fitting separatrix')

        timeout_pool = 90
        pool = Pool(cpu_count())
        out = pool.map_async(descu, [(self.rscat[jt][0], self.zscat[jt][0], nmom, self.time[jt], self.error[jt]) for jt in range(nt_in)]).get(timeout_pool)
        pool.close()
        pool.join()

        logger.info('Done fitting separatrix')

        self.moments = np.array(out)
        logger.debug(self.moments.shape)

# Discard "bad" time points

        for jt in range(nt_in):
            if self.error[jt] == 0 and np.max(self.moments[jt, :, 0]) <= 0:
                self.error[jt] = 5

        self.r_bnd, self.z_bnd = mom2rz.mom2rz(self.moments[:, :, 0], \
                                               self.moments[:, :, 1], \
                                               self.moments[:, :, 2], \
                                               self.moments[:, :, 3], \
                                               nthe=n_the, endpoint=True)
        for jt in range(nt_in):
            if min(self.z_bnd[jt]) < -1.1:
                self.error[jt] = 6
                self.r_bnd[jt, :] = 0
                self.z_bnd[jt, :] = 0
        ind_bad = (self.error > 0)
        self.moments[ind_bad, :, :] = 0
        self.ind_ok = ~ind_bad
