import os, logging
import numpy as np
from trview import fconf

try:
    import Tkinter as tk
    import ttk
except:
    import tkinter as tk
    from tkinter import ttk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as nt2tk
except:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as nt2tk
from matplotlib.figure import Figure

col = 2*['#c00000', '#00c000', '#0000c0', '#b0b000', '#b000b0', '#00b0b0']
fsize = 14

logger = logging.getLogger('trview.plot_power')


def plot_power(tarr, parr, nshot, powframe=None, y_ticks=[], col='#c00000', ylbl=''):

    if powframe is None:
        powframe = tk.Toplevel()
        powframe.title('Several signals')
        powframe.geometry('1200x960')

    figframe  = ttk.Frame(powframe)
    toolframe = ttk.Frame(powframe)
    for frame in toolframe, figframe:
        frame.pack(side=tk.BOTTOM, fill=tk.BOTH, expand=1)

    fig = Figure(figsize=(8.2, 8.2), dpi=100)
    can = FigureCanvasTkAgg(fig, master=figframe)
    can._tkcanvas.pack(side=tk.TOP, fill=tk.BOTH, expand=1)
    fig.subplots_adjust(left=0.1, bottom=0.1, right=0.95, top=0.92, hspace=0)
    fig.text(.5, .95, '#%d' %nshot, ha='center')

    ymax = int(1e-6*np.max(parr))*1e6 + 1e6
    nsrc = parr.shape[1]
    for jx in range(nsrc):
        ax = fig.add_subplot(8, 1, jx+1)
        ax.plot(tarr, parr[:, jx], color=col)
        ax.set_ylabel('%s%d' %(ylbl, jx+1), fontsize=fsize)
        ax.set_ylim([0, ymax])
#        ax.set_yticks(y_ticks)
        ax.ticklabel_format(axis='y', style='sci', scilimits=(-4, 4))
        ax.yaxis.major.formatter._useMathText = True
        if jx < nsrc-1:
            ax.tick_params(axis='x', which='both', bottom='on', top='on', labelbottom='off')

    ax.set_xlabel('Time [s]', fontsize=fsize)
    can.mpl_connect('button_press_event', fconf.on_click)
    toolbar = nt2tk(can, toolframe)
    toolbar.update()
