import os, logging
import numpy as np
try:
    import Tkinter as tk
    import ttk
except:
    import tkinter as tk
    from tkinter import ttk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as nt2tk
except:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as nt2tk
from matplotlib.figure import Figure

logger = logging.getLogger('trview.plot_sig')
logger.setLevel(logging.INFO)


col = 2*['#c00000', '#00c000', '#0000c0', '#b0b000', '#b000b0', '#00b0b0']
fsize = 14


def plot_sig(sig1d, sigframe=None):

    if sigframe is None:
        sigframe = tk.Toplevel()
        sigframe.title('Several signals')

    figframe  = ttk.Frame(sigframe)
    toolframe = ttk.Frame(sigframe)
    for frame in (toolframe, figframe):
        frame.pack(side=tk.BOTTOM, fill=tk.BOTH, expand=1)
    ymax = figframe.winfo_screenheight()
    height = min(800, int(0.88*ymax)) 
    fig_hei = height - 100
    fig_dpi = 100
    logger.debug(fig_hei, fig_dpi,  fig_hei/fig_dpi)
    sigfig = Figure(figsize=(fig_hei/fig_dpi, fig_hei/fig_dpi), dpi=100)
    sig_can = FigureCanvasTkAgg(sigfig, master=figframe)
    sig_can._tkcanvas.pack(side=tk.TOP, fill=tk.BOTH, expand=1)
    sigfig.subplots_adjust(left=0.1, bottom=0.1, right=0.95, top=0.92, hspace=0)

#---------------------
# Ipl, Bt, Uloop, Zeff
#---------------------

    n_ct = 1
    n_sub = len(sig1d.__dict__.keys())
    tbeg = 0.
    tend = 10.
    for key, data in sig1d.__dict__.items():
        logger.debug(key)
        tbeg = min(tbeg, data.t_ds[0])
        tend = max(tend, data.t_ds[-1])

    for key, data in sig1d.__dict__.items():

        diag, sig = data.diagsig.split(':')
        logger.debug('%s: %s', diag, sig)
        if data.unit is None:
            ylab = key
        else:
            ylab = '%s [%s]' %(key, data.unit)
        ax = sigfig.add_subplot(n_sub, 1, n_ct)
        ax.plot(data.t_ds, data.data_ds, color=col[n_ct])
        ax.set_xlim([tbeg, tend])
        ax.set_ylabel(ylab, fontsize=fsize)
        ax.tick_params(axis='x', which='both', bottom='on', top='on', labelbottom='off')
        ax.tick_params(axis='y', which='both', left='on', right='on')
        ax.ticklabel_format(axis='y', style='sci', scilimits=(0, 0))
        ax.yaxis.major.formatter._useMathText = True
        n_ct += 1
    ax.tick_params(axis='x', which='both', bottom='on', top='on', labelbottom='on')
    ax.set_xlabel('Time [s]', fontsize=fsize)

    sigfig.text(.5, .95, '#%d' %data.shot, ha='center')
    sig_can.draw()

# Toolbar

    toolbar = nt2tk(sig_can, toolframe)
    toolbar.update()

    if __name__ == '__main__':
        sigframe.mainloop()
