try:
    import Tkinter as tk
    import ttk
except:
    import tkinter as tk
    from tkinter import ttk
import logging
import numpy as np
from trview import read_prof, time_slice, vprof, tkmpl

logger = logging.getLogger('trview.profiles')

pre_list = ['I', 'E', 'N', 'V']
frname = {'I': 'Ti', 'E': 'Te', 'N': 'ne', 'V': 'vrot'}


class EXP_PROF:


    def __init__(self, gpar, equ, nb_prof, sflists, fit_d):


        self.raw_data = {}
        self.sfl_d = {}
        for prefix in pre_list:
            self.sfl_d[prefix] = []
            for sflist in sflists[prefix]: #sf is like 'AUGD:CEZ:Ti:0'
                raw_d = read_prof.dd2read(sflist, equ, gpar)
                if raw_d is not None:
# Replace edition 0 with actual edition number
                    sf_list = sflist.split(':')
                    sfl = ':'.join(sf_list[:-1])
                    sfl += ':%d' %raw_d['ed']
                    self.raw_data[sfl] = raw_d
                    self.sfl_d[prefix].append(sfl)
# Map to common time base, discard profiles close to ELMs
        self.slice(gpar)
# Initial fit
        self.fit(gpar, sflists, fit_d, nb_prof)


    def slice(self, gpar):

# Create wished time grid

        tbeg = 10.
        tend = 0.
        for raw_d in self.raw_data.values():
            tbeg = min(tbeg, raw_d['tgrid'][0])
            tend = max(tend, raw_d['tgrid'][-1])
        tbeg = max(tbeg, gpar.tbeg)
        tend = min(tend, gpar.tend)

        if tend >= tbeg + gpar.dt_prof:
            nt = int((tend - tbeg)/ gpar.dt_prof)
            self.tgrid = tbeg + gpar.dt_prof*np.arange(nt + 1)
        else:
            self.tgrid = np.atleast_1d(tbeg)

# Store mapped data into dict

        self.red_data = {}
        for prefix in pre_list:
            self.red_data[prefix] = {}
            for sflist in self.sfl_d[prefix]:
                raw_d = self.raw_data[sflist]
                self.red_data[prefix][sflist] = {}
                x_red, y_red, err_red = time_slice.map2tgrid( \
                raw_d, self.tgrid, nshot=gpar.shot, noELMs=gpar.no_elm, dt_out=gpar.dt_prof)
                self.red_data[prefix][sflist]['rhop']     = x_red
                self.red_data[prefix][sflist]['data']     = y_red
                self.red_data[prefix][sflist]['data_err'] = err_red
                self.red_data[prefix][sflist]['tgrid']    = self.tgrid
                self.red_data[prefix][sflist]['unit']     = raw_d['unit']


    def fit(self, gpar, sflists, fit_d, nb_prof):

        self.pr_obj = gpar

# Recursive spline or Gauss
        nshot = self.pr_obj.shot
        n_spl = self.pr_obj.nrho
        self.pr_obj.time = self.tgrid
        self.pr_obj.rho  = np.linspace(0, 1, n_spl)
        self.pr_obj.prof = {}

        for prefix in pre_list:

            if len(self.red_data[prefix].keys()) == 0:
                logger.error('\nNo diagnostic data for %s among selected shotfiles', frname[prefix] )
                logger.error(sflists[prefix])
                logger.error('Skipping\n')
                continue
#                return
            tab = ttk.Frame(nb_prof, name=frname[prefix].lower())
            nb_prof.add(tab, text=frname[prefix])

            fit_method = fit_d[prefix].get().strip()
            if fit_method in ('rec_spl', 'gauss'):
                logger.info('Fit method = %s', fit_method)
                obj = vprof.MPLWID(nshot, n_spl, self.red_data[prefix], method=fit_method, myframe=tab)
            else:
# 2D method
                obj = tkmpl.DataFit(nshot, n_spl, self.red_data[prefix], myframe=tab, eq_exp=self.pr_obj.eq_exp, eq_diag=self.pr_obj.eq_diag, eq_ed=self.pr_obj.eq_ed)

            if hasattr(obj, 'yfit_rhop'):
                self.pr_obj.prof[prefix] = obj
                self.pr_obj.prof[prefix].sflists = sorted(self.red_data[prefix].keys())
                self.pr_obj.prof[prefix].units   = self.red_data[prefix][obj.sflists[0]]['unit']

        logger.info('Done fitting kinetic profiles')
