import os, sys, logging
import ctypes as ct
import numpy as np

logger = logging.getLogger('trview.rec_spline1d')

gsl_con = os.getenv('GSLCONTRIB_LIB')
if gsl_con is None:
    gsl_con = '/afs/ipp/home/g/git/gsl_contrib/lib/libgsl-contrib.so'
libgsl = ct.cdll.LoadLibrary(gsl_con)

c_dbl_p = ct.POINTER(ct.c_double)


def spline_rec1d(x_exp, y_exp, x_out, sigmas=1, fit_tol=1, y_err=None):

    n_options = 1
    nx_out = len(x_out)

# Declaring+initialising ctypes scalars
    c_err = ct.c_long(0)
    n_spl = ct.c_long(nx_out)
    c_chisq = ct.c_double(1)
    options = ct.c_long(n_options) #options=1,2,3 force der at min(xspl),max(xspl),both
    maxiter = ct.c_long(100)

# Declaring ctypes arrays
#    opt_values = (ct.c_double * 2)()
    opt_values = (ct.c_double * 1)()

# Initialising arrays
    opt_values[0] = ct.c_double(0.)     #derivative at min(xspl)
#    opt_values[1] = ct.c_double(-0.5)   #derivative at max(xspl)
    c_sigma = ct.c_double(sigmas)

# Auxiliary parameters
    xspl = x_out.ctypes.data_as(c_dbl_p)

    x_exp = np.array(x_exp, dtype=np.float64)
    y_exp = np.array(y_exp, dtype=np.float64)
    nx = len(x_exp)
    if y_err is None:
        err = np.repeat(fit_tol, nx).astype(np.float64, copy=False)
    else:
        ind_zero = (y_exp == 0) 
        err = np.array(fit_tol*y_err, dtype=np.float64)

    nx_out = n_spl.value
    y_rec = np.zeros(nx_out, dtype = np.float64) # spline fit
    c_nx = ct.c_long(nx)

    if gsl_con is None:
        logger.error('Env variable GSLCONTRIB_LIB not defined')
        logger.error('Execute "module load trview" and run rabbit.py again')
        sys.exit()

    if not os.path.isfile(gsl_con) :
        logger.error('Dir for GSL-based spline fit not found: %s', gsl_con)
        sys.exit()

    try:
        libgsl.gsl_rec_spl1d( \
            c_err, c_sigma, c_nx, \
            x_exp.ctypes.data_as(c_dbl_p), \
            y_exp.ctypes.data_as(c_dbl_p), \
            err.ctypes.data_as(c_dbl_p), \
            n_spl, xspl, \
            maxiter, options, opt_values, \
            c_chisq, \
            y_rec.ctypes.data_as(c_dbl_p))
    except:
        logger.error('Problems with spline_rec fit, returning zero arrays')

    return y_rec, y_exp
