import logging
import numpy as np

logger = logging.getLogger('trview.red_time')

# Reduce time array size, taking points before AND after a big step

def red_time(min_step, arr):

    nt = arr.shape[0]
    if arr.ndim == 1:
        tdiff = np.abs(np.diff(arr) > min_step)
    elif arr.ndim == 2:
        tdiff = np.max(np.abs(np.diff(arr, axis=0)), axis=1)
    else:
        logger.error('Too many dimensions for input array', arr.ndim)
        return None

    (ind_red, ) = np.where(tdiff > min_step)
# Add points after the significant step
    ind = np.append(ind_red, np.minimum(ind_red + 1, nt - 1))
# Always include first and last time point
    ind = np.sort(np.append([0, nt-1], ind))

    return np.unique(ind)
