import os, fnmatch, shutil, logging
import numpy as np
import aug_sfutils as sf
from trview import ufiles, tr_path

logger = logging.getLogger('trview.wrnml')

aug_temp = '/afs/ipp/home/t/transp/tr_client/AUGD/templates'


def wr_arr(lbl, arr, fmt='%6.1f'):

    logger.info(lbl)
    arr_nml = '%-14s = ' %lbl
    for val in arr[:-1]:
        arr_nml += (fmt %val)
        arr_nml += ', '
    arr_nml += (fmt %arr[-1])
    arr_nml +='\n'

    return arr_nml


def sym_prof(uname):

    uf = ufiles.UFILE(fin=uname)
    rho_lbl = uf.Y['label'].strip()
    if rho_lbl == 'rho_pol':
        sym_prof = -7
    elif rho_lbl == 'rho_tor':
        sym_prof = -5
    return sym_prof


def cpuf(runid):

    tr = tr_path.TR_PATH(runid)
    nshot = int(runid[:-3])
    s5shot = '%05d' %nshot
    preext_io = [
            ('M'  , 'MRY'), 
            ('SM' , 'CUR'), 
            ('S'  , 'RBZ'), 
            ('U'  , 'VSF'), 
            ('P'  , 'NBI'), 
            ('P'  , 'ICH'), 
            ('P'  , 'ECH'), 
            ('ICR', 'MINL'), 
            ('ICR', 'MINR'), 
            ('PHITR', 'ECH'), 
            ('Q'  , 'QPR'),
            ('THETR', 'ECH'), 
            ('Z'  , 'ZEF')
        ]

    for pre, ext in preext_io:
        ufile = '%s%s.%s'     %(pre, s5shot, ext)
        source = '%s/%s'      %(tr.udb    , ufile)
        target = '%s/%s'      %(tr.run_dir, ufile)
        source2 = '%s/%s_AVG' %(tr.udb    , ufile)
        target2 = '%s/%s_AVG' %(tr.run_dir, ufile)
        if os.path.isfile(source):
            logger.info('cp %s %s' %(source, target))
            shutil.copy2(source, target)
        if os.path.isfile(source2):
            logger.info('cp %s %s' %(source2, target2))
            shutil.copy2(source2, target2)

# Profiles
    for fudb in os.listdir(tr.udb):
        for pre in ('E', 'I', 'N', 'V', 'ANGF'):
            if fnmatch.fnmatch(fudb, '%s%s.*' %(pre, s5shot)):
                try:
                    a = int(fudb[-1])
                except:
                    logger.info(fudb)
                    source = '%s/%s' %(tr.udb, fudb)
                    target = '%s/%s' %(tr.run_dir, fudb)
                    logger.info('cp %s %s' %(source, target))
                    shutil.copy2(source, target)


def w_nml(gpar, ion_d, ext_d, nbi, ech=None, icr=None):


    lev_geo = '6' # 6 computed, 8 prescribed
    mcrf  = gpar.mcrf
    nshot = gpar.shot
    s5shot = '%05d' %nshot
    lett = 'A'
    nrun = 1
    runid = '%sA01' %s5shot
    tr = tr_path.TR_PATH(runid)
    nml_out = tr.fnml
    while os.path.isfile(nml_out):
        nrun +=  1
        runid = '%sA%02d' %(s5shot, nrun)
        tr = tr_path.TR_PATH(runid)
        nml_out = tr.fnml

    os.system('mkdir -p %s' %tr.udb)
    os.system('mkdir -p %s' %tr.run_dir)

    temp_nl = '%s/augTR.DAT' %aug_temp
    u_nl = open(temp_nl, 'r')
    nml = u_nl.read()
    u_nl.close()

#======================== 
# Append NBI, ECRH, ICRH
#======================== 

    s5shot = '%05d' %nshot
    pfile = '%s/P%s' %(tr.udb, s5shot)
    pnbi = pfile + '.NBI'
    pech = pfile + '.ECH'
    pich = pfile + '.ICH'
    if os.path.isfile(pnbi):
        tr_nbi_nml = nbi2nml(nbi)
        logger.info('Appending NBI to namelist')
        nml += '\n' + tr_nbi_nml
    else:
        logger.info('%s not found!!' %pnbi)
    if os.path.isfile(pech) and ech is not None:
        tr_ech_nml = ech2nml(ech)
        logger.info('Appending ECRH to namelist')
        nml += '\n' + tr_ech_nml
    if os.path.isfile(pich) and icr is not None:
        tr_ich_nml = ich2nml(icr, mcrf)
        logger.info('Appending ICRH to namelist')
        nml += '\n' + tr_ich_nml

#================
# Replace values
#================

# Fields' signs

    ipl_sign = 'T'
    btf_sign = 'F'
    fpc = sf.SFREAD(nshot, 'FPC')
    if fpc.status:
        ipl = fpc.getobject('IpiFP')
        btf = fpc.getobject('BTF')
        if np.sum(ipl) < 0:
            ipl_sign = 'F'
        if np.sum(btf) > 0:
            btf_sign = 'T'

    nml = nml.replace('$IPL_CCW', ipl_sign)
    nml = nml.replace('$BT_CCW' , btf_sign)
    nml = nml.replace('$AMAIN', '%8.4f' %ion_d['Amain'])
    nml = nml.replace('$ZMAIN', '%8.4f' %ion_d['Zmain'])
    nml = nml.replace('$AIMP' , '%8.4f' %ion_d['Aimp' ])
    nml = nml.replace('$ZIMP' , '%8.4f' %ion_d['Zimp' ])

# GUI entries
    tres_in = min(gpar.tres, 0.01)
    nml = nml.replace('$TGRID' , '%8.4f' %tres_in)
    nml = nml.replace('$TRES'  , '%8.4f' %tres_in)
    nml = nml.replace('$TBEG'  , '%8.4f' %gpar.tbeg)
    nml = nml.replace('$TEND'  , '%8.4f' %gpar.tend)
    nml = nml.replace('$DTBEAM', '%8.4f' %gpar.dtbeam)

    nml = nml.replace('$SHOT', '%d' %gpar.shot)
    nml = nml.replace('$NMC' , '%d' %gpar.nmc)

# Profiles

    if 'E' not in ext_d.keys():
        raise Exception('Missing Te profile')
    if 'N' not in ext_d.keys():
        raise Exception('Missing ne profile')

    sym_txt = ''
    ext_txt = ''

# Te

    ext = ext_d['E']
    ext_txt += "preter = 'E'\n"  
    ext_txt += "extter = '%s'\n" %ext 
    uname = '%s/E%s.%s' %(tr.udb, s5shot, ext)
    if os.path.isfile(uname):
        sym_txt += 'nriter = %d\n' %sym_prof(uname)
        sym_txt += 'nsyter = 0\n'

# ne

    ext = ext_d['N']
    ext_txt += "prener = 'N'\n"
    ext_txt += "extner = '%s'\n" %ext 
    uname = '%s/N%s.%s' %(tr.udb, s5shot, ext)
    if os.path.isfile(uname):
        sym_txt += 'nriner = %d\n' %sym_prof(uname)
        sym_txt += 'nsyner = 0\n'
    
# Ti

    ti_txt = ''
    if 'I' in ext_d.keys():
        ext = ext_d['I']
        uname = '%s/I%s.%s' %(tr.udb, s5shot, ext)
        if os.path.isfile(uname):
            ti_txt += 'nlti2 = T\n'
            ti_txt += 'fiefac = 1.0\n'
            ti_txt += 'giefac = 0.0\n'
            ext_txt += "preti2 = 'I'\n"  
            ext_txt += "extti2 = '%s'\n" %ext 
            sym_txt += 'nriti2 = %d\n' %sym_prof(uname)
            sym_txt += 'nsyti2 = 0\n'
        else:    # Ti = Te
            ti_txt += 'nlti2 = F\n'
            ti_txt += 'fiefac = 0.0\n'
            ti_txt += 'giefac = 1.0\n'
    else:    # Ti = Te
        ti_txt += 'nlti2 = F\n'
        ti_txt += 'fiefac = 0.0\n'
        ti_txt += 'giefac = 1.0\n'
        
# vtor

    vt_txt = ''

    if 'ANGF' in ext_d.keys():
        ext = ext_d['ANGF']
        uname = '%s/ANGF%s.%s' %(tr.udb, s5shot, ext)
        if os.path.isfile(uname):
            vt_txt += 'nlvphi = T\n'
            vt_txt += 'nlomgvtr = T\n'
            ext_txt += "preomg = 'ANGF'\n"  
            ext_txt += "extomg = '%s'\n" %ext 
            sym_txt += 'nriomg = %d\n' %sym_prof(uname)
            sym_txt += 'nsyomg = 0\n'
    elif 'V' in ext_d.keys():
        ext = ext_d['V']
        uname = '%s/V%s.%s' %(tr.udb, s5shot, ext)
        if os.path.isfile(uname):
            vt_txt += 'nlvphi = T\n'
            vt_txt += 'nlomgvtr = T\n'
            ext_txt += "prevp2 = 'V'\n"  
            ext_txt += "extvp2 = '%s'\n" %ext 
            sym_txt += 'nrivp2 = %d\n' %sym_prof(uname)
            sym_txt += 'nsyvp2 = 0\n'
    else:    # No rotation
        vt_txt += 'nlvphi = F\n'
        
    nml = nml.replace('$ti_txt' , ti_txt)
    nml = nml.replace('$vt_txt' , vt_txt)
    nml = nml.replace('$ext_txt', ext_txt)
    nml = nml.replace('$sym_txt', sym_txt)

# equil

    uname = '%s/M%s.MRY' %(tr.udb, s5shot)
    if os.path.isfile(uname):
        fu = open(uname, 'r')
        eq = {}
        for line in fu.readlines():
            for eqstr in ('exp', 'dia', 'ed'):
                if eqstr in line:
                    eq[eqstr] = line.split('=')[1].strip()
        for key, val in eq.items():
            nml = nml.replace('$eq_' + key, val)

    nml = nml.replace('$SHOT', s5shot)
    nml = nml.replace('$LEVGEO', lev_geo)

    if lev_geo == '8':
        nml = nml.replace('mry', 'mmx')
        nml = nml.replace('MRY', 'MMX')

    if mcrf:
        nml = nml.replace('$EBDMAX', '0.500000E+06')
    else:
        nml = nml.replace('$EBDMAX', '0.150000E+06')

#===== 
# FBM
#===== 

    if hasattr(gpar, 'avgtim'):
        avgtim = gpar.avgtim
        fbm_source = '!$FBM'
        fbm_target = ''
        tbeg = gpar.tbeg
        tend = gpar.tend

        if len(gpar.outtim) > 0:
            fbm_target += 'MTHDAVG=2\n'
            fbm_target += 'AVGTIM=%8.4f\n' %avgtim
            fbm_target += 'NLDEP0_GATHER=T\n'
            for jfb, tfbm in enumerate(gpar.outtim):
                fbm_target += 'OUTTIM(%d) =%6.3f\n' %(jfb + 1, tfbm)

        nml = nml.replace(fbm_source, fbm_target)

    u_out = open(nml_out, 'w')
    u_out.write(nml)
    u_out.close()

    cpuf(runid)

    logger.info('Output namelist %s', nml_out)

    return runid


def nbi2nml(nbi):

# TRANSP NBI namelist

    if not hasattr(nbi, 'rtcena'):
        nbi.aug_tr()

    tr_nbi_temp =  '%s/nbiTR.DAT' %aug_temp
    if nbi.shot <=  14050:
        tr_nbi_temp =  '%s/nbi_oldTR.DAT' %aug_temp
        logger.warning('Old beam geometry, using %s template', tr_nbi_temp)

    u_temp = open(tr_nbi_temp, 'r')
    tr_nbi_nml = u_temp.read()
    u_temp.close()

    nnbi = len(nbi.einja)

    for jx in range(nnbi):
        source = '$EINJA%d' %(jx+1)
        target = '%10.3f' % nbi.einja[jx]
        tr_nbi_nml = tr_nbi_nml.replace(source, target)

        source = '$FFULLA%d' %(jx+1)
        target = '%10.3f' % nbi.ffulla[jx]
        tr_nbi_nml = tr_nbi_nml.replace(source, target)

        source = '$FHALFA%d' %(jx+1)
        target = '%10.3f' % nbi.fhalfa[jx]
        tr_nbi_nml = tr_nbi_nml.replace(source, target)

    if nbi.shot >  14050:
        for jx in (5, 6):
            source = '$XLBTNA%d' %(jx+1)
            target = '%10.3f' % nbi.xlbtna[jx]
            tr_nbi_nml = tr_nbi_nml.replace(source, target)

            source = '$XLBAPA%d' %(jx+1)
            target = '%10.3f' % nbi.xlbapa[jx]
            tr_nbi_nml = tr_nbi_nml.replace(source, target)

            source = '$XYBAPA%d' %(jx+1)
            target = '%10.3f' % nbi.xybapa[jx]
            tr_nbi_nml = tr_nbi_nml.replace(source, target)

# ASTRA nbi

    return tr_nbi_nml


def ech2nml(ech):

# git 07.04.2014
# Wrong interpretation of toroidal angle from shotfile!! phi_TORAY = 180-phi_SF, not +

    nshot = ech.shot

    if not hasattr(ech, 'p_ecs'):

        return '! No ECRH'

# Mirror angles

    ngy = len(ech.freq)

# Parameter output: TORAY piece of namelist

    tr_ech_nml = """
!=================
! TORBEAM for ECRH
!=================
PREECP='P'
EXTECP='ECH'
NLTORBEAM = T
NLAUNCHECH = 1
"""
    tr_ech_nml += 'NANTECH = %d\n\n' %ngy

    tr_ech_nml += """
! generate files for TORBEAM standalone
!NLTORBEAM_VERBOSE = T
!FE_OUTTIM(1) = 4.605
! control what Zeff is used by TORBEAM. -1. -> Zeff profile (default)
NTOB_NPROFVW = 25
NTOB_MMAX = 150
TOB_XZEFF = -1

PREECA = 'THETR'
EXTECA = 'ECH'
PREECB = 'PHITR'
EXTECB = 'ECH'
"""
    if hasattr(ech, 'freq'):    
        tr_ech_nml +=  wr_arr('FREQECH', ech.freq, fmt='%10.3e')
    else:
        tr_ech_nml += 'FREQECH error\n'

    tr_ech_nml += wr_arr('XECECH', 100.*ech.R)
    tr_ech_nml += wr_arr('ZECECH', 100.*ech.z)
    tr_ech_nml += wr_arr('ECB_WIDTH_HOR' , 100.*ech.width_y)
    tr_ech_nml += wr_arr('ECB_WIDTH_VERT', 100.*ech.width_z)
    tr_ech_nml += wr_arr('ECB_CURV_HOR'  , 100.*ech.curv_y)
    tr_ech_nml += wr_arr('ECB_CURV_VERT' , 100.*ech.curv_z)
    logger.debug(ech.rfmod)
    tr_ech_nml += wr_arr('RFMODECH', ech.rfmod)

    return tr_ech_nml


def ich2nml(icr, mcrf):

    nshot = icr.shot
    tr_ich_temp =  '%s/ichTR.DAT' %aug_temp

    u_temp = open(tr_ich_temp, 'r')
    tr_ich_nml = u_temp.read()
    u_temp.close()

    source = '$FRQicha'
    target = 'FRQicha = '
    for freq in icr.freq:
        target += '%12.5e, ' %freq
    target = target[:-2]
    tr_ich_nml = tr_ich_nml.replace(source, target)
    if mcrf: #MC kick-operator
        for target in ('NLFI_MCRF', 'NLFI_ORBRZ', 'EDBFAC', 'NZNBME'):
            source = '!'+target
            tr_ich_nml = tr_ich_nml.replace(source, target)
    return tr_ich_nml
